@@ -1,13 +1,9 @@
 from os import path
 
-from . import ffmpeg, configure
+from . import ffmpegprocess, configure
 
 
-def transcode(
-    input_url,
-    output_url,
-    **options
-):
+def transcode(input_url, output_url, progress=None, capture_log=None, **options):
     """Transcode a media file to another format/encoding
 
     :param input_url: url/path of the input media file
@@ -20,6 +16,11 @@ def transcode(
     :type output_options: dict, optional
     :param global_options: dict of user-defined global options, defaults to None. each key is FFmpeg option argument without leading '-' with trailing stream specifier if needed. For flag options, set their values to None.
     :type global_options: dict, optional
+    :param progress: progress callback function, defaults to None
+    :type progress: callable object, optional
+    :param capture_log: True to capture log messages on stderr, False to send
+                    logs to console, defaults to None (no show/capture)
+    :type capture_log: bool, optional
     :param \\**options: other keyword options (see :doc:`options`).
     :type \\**options: dict, optional
     :return: returncode of FFmpeg subprocess
@@ -28,7 +29,13 @@ def transcode(
 
     """
 
-    args = configure.input_timing({},input_url, **options)
+    input_url, stdin, input = configure.check_url(input_url, False)
+    output_url, stdout, _ = configure.check_url(output_url, True)
+
+    if input is not None:
+        input_url = "-"
+
+    args = configure.input_timing({}, input_url, **options)
 
     configure.codec(args, output_url, "v", prefix="video_", **options)
     configure.codec(args, output_url, "a", prefix="audio_", **options)
@@ -56,17 +63,24 @@ def transcode(
     configure.global_options(args, **options)
 
     if "input_options" in options:
-        configure.merge_user_options(args, "input", options['input_options'])
+        configure.merge_user_options(args, "input", options["input_options"])
 
     if "output_options" in options:
-        configure.merge_user_options(args, "output", options['output_options'])
+        configure.merge_user_options(args, "output", options["output_options"])
 
     if "global_options" in options:
-        configure.merge_user_options(args, "global", options['global_options'])
+        configure.merge_user_options(args, "global", options["global_options"])
 
     # TODO run async and monitor stderr for better error handling
     try:
-        ffmpeg.run_sync(args, stdout=None, stderr=None)
+        ffmpegprocess.run(
+            args,
+            progress=progress,
+            capture_log=capture_log,
+            stdin=stdin,
+            stdout=stdout,
+            input=input,
+        )
     except Exception as e:
         if configure.is_forced(args) and path.isfile(output_url):
             raise e
