@@ -367,7 +367,7 @@ def array_to_audio_input(
     :rtype: tuple(str, dict)
     """
 
-    spec = spec_stream(stream_id, "a")
+    spec = "" if stream_id is None else ":" + spec_stream(stream_id, "a")
 
     if codec is None:
         codec, sample_fmt = get_audio_format(data.dtype)
@@ -376,17 +376,17 @@ def array_to_audio_input(
         ndim = data.ndim
         if ndim < 1 or ndim > 2:
             raise Exception("audio data array must be 1d or 2d")
-        channels = shape[-1] if ndim > 1 else shape[0] if ndim > 0 else 1
+        channels = shape[-1] if ndim > 1 else 1
     elif sample_fmt is None or channels is None:
         raise Exception(
             "configuring audio input with a custom codec requires `sample_fmt` and `channels` to be also specified."
         )
 
     opts = {
-        f"c:{spec}": codec,
-        f"ac:{spec}": channels,
-        f"ar:{spec}": rate,
-        f"sample_fmt:{spec}": sample_fmt,
+        f"c{spec or ':a'}": codec,
+        f"ac{spec}": channels,
+        f"ar{spec}": rate,
+        f"sample_fmt{spec}": sample_fmt,
     }
     input = ("-", opts)
 
